from __future__ import annotations

from typing import NamedTuple


class Login_Responses(NamedTuple):

    AccessToken: str
    """
    The token to be used along with accessing a Command or Property on a SiLA Server.
    """

    TokenLifetime: int
    """
    The lifetime (in seconds) of the provided access token as the maximum validity period after the last SiLA Server
        request.
    """


class Logout_Responses(NamedTuple):

    pass
