class FeatureImplementationBase:
    def start(self) -> None:
        """This method is called by the SiLA server when it starts serving this feature implementation"""
        pass

    def stop(self) -> None:
        """This method is called by the SiLA server when it starts serving this feature implementation"""
        pass
